@@ -5,8 +5,9 @@
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <sven@convergence.de> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -81,8 +82,8 @@ DECLARE_INTERFACE( IFusionSoundMusicProvider )
  * call this before doing anything else. Removes all options used by FusionSound from argv.
  */
 DFBResult FusionSoundInit(
-                           int   *argc,   /* pointer to main()'s argc */
-                           char **argv[]  /* pointer to main()'s argv */
+                           int    *argc,   /* pointer to main()'s argc */
+                           char *(*argv[]) /* pointer to main()'s argv */
                          );
 
 /*
@@ -132,6 +133,36 @@ const char *FusionSoundErrorString(
  */
 const char *FusionSoundUsageString( void );
 
+
+#define FS_SOUND_DRIVER_INFO_NAME_LENGTH     40
+#define FS_SOUND_DRIVER_INFO_VENDOR_LENGTH   60
+#define FS_SOUND_DRIVER_INFO_URL_LENGTH     100
+#define FS_SOUND_DRIVER_INFO_LICENSE_LENGTH  40
+
+/*
+ * Description of the sound driver.
+ */
+typedef struct {
+     int  major;                                        /* Major version. */
+     int  minor;                                        /* Minor version. */
+     
+     char name[FS_SOUND_DRIVER_INFO_NAME_LENGTH];       /* Driver name. */
+     char vendor[FS_SOUND_DRIVER_INFO_VENDOR_LENGTH];   /* Driver vendor. */
+     char url[FS_SOUND_DRIVER_INFO_URL_LENGTH];         /* Driver URL. */
+     char license[FS_SOUND_DRIVER_INFO_LICENSE_LENGTH]; /* Driver license. */
+} FSSoundDriverInfo;
+
+#define FS_SOUND_DEVICE_INFO_NAME_LENGTH    96
+
+/*
+ * Description of the sound device.
+ */
+typedef struct {
+     char name[FS_SOUND_DEVICE_INFO_NAME_LENGTH];       /* Device name. */
+     
+     FSSoundDriverInfo driver;
+} FSDeviceDescription;
+
 /*
  * @internal
  *
@@ -153,12 +184,12 @@ const char *FusionSoundUsageString( void );
 /*
  * The sample format is the way of storing audible information.
  *
- * 8, 16 and 32 bit samples are always stored in <b>native endian</b>, while
- * 24 bit samples are stored in <b>little endian</b>. This keeps the library and
- * applications simple and clean. Always access sample buffers like arrays of
- * 8, 16 or 32 bit integers depending on the sample format, unless data is
- * written with endianness being taken care of. This does not excuse from endian
- * conversion that might be necessary when reading data from files.
+ * 16, 24 and 32 bit samples are always stored in <b>native endian</b>.
+ * This keeps the library and applications simple and clean. 
+ * Always access sample buffers like arrays of 8, 16 or 32 bit integers
+ * depending on the sample format, unless data is written with endianness
+ * being taken care of. This does not excuse from endian conversion
+ * that might be necessary when reading data from files.
  */
 typedef enum {
      FSSF_UNKNOWN        = 0x00000000, /* Unknown or invalid format. */
@@ -169,15 +200,18 @@ typedef enum {
      /* Signed 16 bit (native endian). */
      FSSF_S16            = FS_SAMPLEFORMAT( 1, 16, 16, 1 ),
 
-     /* Signed 24 bit (little endian). */
+     /* Signed 24 bit (native endian). */
      FSSF_S24            = FS_SAMPLEFORMAT( 2, 24, 24, 1 ),
 
      /* Signed 32 bit (native endian). */
      FSSF_S32            = FS_SAMPLEFORMAT( 3, 32, 32, 1 ),
+     
+     /* Floating-point 32 bit (native endian). */
+     FSSF_FLOAT          = FS_SAMPLEFORMAT( 4, 32, 32, 1 ),
 } FSSampleFormat;
 
 /* Number of defined sample formats */
-#define FS_NUM_SAMPLEFORMATS  4
+#define FS_NUM_SAMPLEFORMATS  5
 
 /* These macros extract information about the sample format. */
 #define FS_SAMPLEFORMAT_INDEX( fmt )  (((fmt) & 0x0000000F)      )
@@ -188,7 +222,7 @@ typedef enum {
 
 #define FS_SAMPLEFORMAT_DEPTH( fmt )  (((fmt) & 0x0003f800) >> 11)
 
-#define FS_SIGNED_SAMPLEFORMAT( fmt ) (((fmt) & 0x00040000) ? 1 : 0)
+#define FS_SIGNED_SAMPLEFORMAT( fmt ) (((fmt) & 0x00040000) !=  0)
 
 
 /*
@@ -248,10 +282,31 @@ typedef struct {
  * Information about an IFusionSoundMusicProvider.
  */
 typedef enum {
-     FMCAPS_BASIC      = 0x00000000,  /* basic ops (PlayTo, Stop)       */
-     FMCAPS_SEEK       = 0x00000001,  /* supports SeekTo                */
+     FMCAPS_BASIC      = 0x00000000,  /* Basic ops (PlayTo, Stop).                */
+     FMCAPS_SEEK       = 0x00000001,  /* Supports SeekTo.                         */
+     FMCAPS_RESAMPLE   = 0x00000002,  /* Supports audio resampling.               */
+     FMCAPS_HALFRATE   = 0x00000004,  /* Supports decoding at half original rate. */
 } FSMusicProviderCapabilities;
 
+/*
+ * Information about the status of a music provider.
+ */
+typedef enum {
+     FMSTATE_UNKNOWN   = 0x00000000, /* Unknown status.       */
+     FMSTATE_PLAY      = 0x00000001, /* Provider is playing.  */
+     FMSTATE_STOP      = 0x00000002, /* Playback was stopped. */
+     FMSTATE_FINISHED  = 0x00000003, /* Playback is finished. */
+} FSMusicProviderStatus;
+
+/*
+ * Flags controlling playback of a music provider.
+ */
+typedef enum {
+     FMPLAY_NOFX       = 0x000000000, /* Normal playback.                        */
+     FMPLAY_LOOPING    = 0x000000001, /* Automatically restart playback as soon
+                                         as end of the current track is reached. */
+} FSMusicProviderPlaybackFlags;
+
 /*
  * Track ID.
  */
@@ -267,13 +322,15 @@ typedef unsigned int FSTrackID;
  * Description of a track provided by a music provider.
  */
 typedef struct {
-     char  artist[FS_TRACK_DESC_ARTIST_LENGTH];     /* Artist */
-     char  title[FS_TRACK_DESC_TITLE_LENGTH];       /* Title */
-     char  album[FS_TRACK_DESC_ALBUM_LENGTH];       /* Album */
-     short year;                                    /* Year */
-     char  genre[FS_TRACK_DESC_GENRE_LENGTH];       /* Genre */
-     char  encoding[FS_TRACK_DESC_ENCODING_LENGTH]; /* Encoding (for example: MPEG Layer-1) */
-     int   bitrate;                                 /* Bitrate in bits/s */
+     char  artist[FS_TRACK_DESC_ARTIST_LENGTH];     /* Artist. */
+     char  title[FS_TRACK_DESC_TITLE_LENGTH];       /* Title. */
+     char  album[FS_TRACK_DESC_ALBUM_LENGTH];       /* Album. */
+     short year;                                    /* Year. */
+     char  genre[FS_TRACK_DESC_GENRE_LENGTH];       /* Genre. */
+     char  encoding[FS_TRACK_DESC_ENCODING_LENGTH]; /* Encoding (for example: MPEG Layer-1). */
+     int   bitrate;                                 /* Bitrate in bits/s. */
+     float replaygain;                              /* ReplayGain level (1.0 by default). */
+     float replaygain_album;                        /* Album ReplayGain level. */
 } FSTrackDescription;
 
 /*
@@ -307,6 +364,16 @@ typedef DFBEnumerationResult (*FSTrackCallback) (
  */
 DEFINE_INTERFACE( IFusionSound,
 
+   /** Information **/
+   
+     /* 
+      * Get a description of the sound device.
+      */
+     DFBResult (*GetDeviceDescription) (
+          IFusionSound           *thiz,
+          FSDeviceDescription    *ret_desc
+     );
+
    /** Buffers **/
 
      /*
@@ -318,9 +385,9 @@ DEFINE_INTERFACE( IFusionSound,
       * are 44kHz, 16 bit (FSSF_S16) with two channels.
       */
      DFBResult (*CreateBuffer) (
-          IFusionSound             *thiz,
-          FSBufferDescription      *desc,
-          IFusionSoundBuffer      **interface
+          IFusionSound               *thiz,
+          const FSBufferDescription  *desc,
+          IFusionSoundBuffer        **interface
      );
 
      /*
@@ -331,9 +398,9 @@ DEFINE_INTERFACE( IFusionSound,
       * size that holds enough samples for one second of playback.
       */
      DFBResult (*CreateStream) (
-          IFusionSound             *thiz,
-          FSStreamDescription      *desc,
-          IFusionSoundStream      **interface
+          IFusionSound               *thiz,
+          const FSStreamDescription  *desc,
+          IFusionSoundStream        **interface
      );
 
      /*
@@ -358,10 +425,10 @@ typedef enum {
                                            looping playback at a time is
                                            supported by the simple playback.
                                            See also <i>CreatePlayback()</i>. */
-     FSPLAY_PAN          = 0x00000002,  /* Use value passed to <i>SetPan()</i>
-                                           to control the balance between left
-                                           and right speakers. */
-     FSPLAY_ALL          = 0x00000003   /* All of these. */
+     FSPLAY_CYCLE        = 0x00000002,  /* Play the whole buffer for one cycle, 
+                                           wrapping at the end. */
+     FSPLAY_REWIND       = 0x00000004,  /* Play reversing sample order. */
+     FSPLAY_ALL          = 0x00000007   /* All of these. */
 } FSBufferPlayFlags;
 
 
@@ -374,10 +441,9 @@ typedef enum {
  *
  * <b>Simple playback</b> is provided by this interface. It includes an
  * unlimited number of non-looping playbacks plus one looping playback at a
- * time. Before <b>starting</b> a playback with <i>Play()</i> the application
- * can <b>adjust the pan value</b> with <i>SetPan()</i> and set the FSPLAY_PAN
- * playback flag. To start the <b>looping</b> playback use FSPLAY_LOOPING. It
- * will <b>stop</b> when the interface is destroyed or <i>Stop()</i> is called.
+ * time. To start the <b>looping</b> playback with <i>Play()</i> use the 
+ * FSPLAY_LOOPING playback flag. It will <b>stop</b> when the interface is
+ * destroyed or <i>Stop()</i> is called.
  *
  * <b>Advanced playback</b> is provided by an extra interface called
  * <i>IFusionSoundPlayback</i> which is created by <i>CreatePlayback()</i>.
@@ -393,26 +459,43 @@ DEFINE_INTERFACE( IFusionSoundBuffer,
       */
      DFBResult (*GetDescription) (
           IFusionSoundBuffer       *thiz,
-          FSBufferDescription      *desc
+          FSBufferDescription      *ret_desc
      );
 
 
+   /** Positioning **/
+   
+     /*
+      * Set the buffer position indicator.
+      *
+      * Set the buffer position indicator (in frames)
+      * affecting subsequent playback and lock for access.
+      */
+     DFBResult (*SetPosition) (
+          IFusionSoundBuffer       *thiz,
+          int                       position
+     );
+     
+     
    /** Access **/
 
      /*
       * Lock a buffer to access its data.
       *
-      * Lock/unlock semantics are weak right now, API will change!
+      * Optionally returns the amount of available frames or bytes
+      * at the current position.
+      *
+      * See also <i>SetPosition()</i>.
       */
      DFBResult (*Lock) (
           IFusionSoundBuffer       *thiz,
-          void                    **data
+          void                    **ret_data,
+          int                      *ret_frames,
+          int                      *ret_bytes
      );
 
      /*
       * Unlock a buffer.
-      *
-      * Lock/unlock semantics are weak right now, API will change!
       */
      DFBResult (*Unlock) (
           IFusionSoundBuffer       *thiz
@@ -421,19 +504,11 @@ DEFINE_INTERFACE( IFusionSoundBuffer,
 
    /** Simple playback **/
 
-     /*
-      * Set panning value.
-      *
-      * The <b>value</b> ranges from -1.0f (left) to 1.0f (right).
-      */
-     DFBResult (*SetPan) (
-          IFusionSoundBuffer       *thiz,
-          float                     value
-     );
-
      /*
       * Start playing the buffer.
       *
+      * Start playing the buffer at the specified position 
+      * (see <i>SetPosition()</i>).
       * There's no limited number of concurrent playbacks, but the simple
       * playback only provides one looping playback at a time.
       *
@@ -501,7 +576,7 @@ DEFINE_INTERFACE( IFusionSoundStream,
       */
      DFBResult (*GetDescription) (
           IFusionSoundStream       *thiz,
-          FSStreamDescription      *desc
+          FSStreamDescription      *ret_desc
      );
 
 
@@ -559,12 +634,23 @@ DEFINE_INTERFACE( IFusionSoundStream,
      /*
       * Flush the ring buffer.
       *
-      * This method stops the playback immediately and discards any buffered data.
+      * This method stops the playback immediately and 
+      * discards any buffered data.
       */
      DFBResult (*Flush) (
           IFusionSoundStream       *thiz
      );
 
+     /*
+      * Drop pending data.
+      *
+      * This method discards any pending input data,
+      * making <i>Write()</i> return as soon as possible.
+      */
+     DFBResult (*Drop) (
+          IFusionSoundStream       *thiz
+     );
+
 
    /** Timing **/
 
@@ -600,6 +686,14 @@ DEFINE_INTERFACE( IFusionSoundStream,
      );
 )
 
+/*
+ * Direction of a playback.
+ */
+typedef enum {
+     FSPD_FORWARD        = +1, /* Forward.  */
+     FSPD_BACKWARD       = -1, /* Backward. */
+} FSPlaybackDirection;
+
 /*
  * <i><b>IFusionSoundPlayback</b></i> represents one concurrent playback and
  * provides full control over the internal processing of samples.
@@ -615,8 +709,8 @@ DEFINE_INTERFACE( IFusionSoundStream,
  * <b>Information</b> provided by <i>GetStatus()</i> includes the current
  * <b>position</b> and whether the playback is <b>running</b>.
  *
- * <b>Parameters</b> provide <b>live</b> control over <b>volume</b>, <b>pan</b>
- * and <b>pitch</b> (speed factor) of the playback.
+ * <b>Parameters</b> provide <b>live</b> control over <b>volume</b>, <b>pan</b>,
+ * <b>pitch</b> (speed factor) and <b>direction</b> of the playback.
  */
 DEFINE_INTERFACE( IFusionSoundPlayback,
 
@@ -713,7 +807,7 @@ DEFINE_INTERFACE( IFusionSoundPlayback,
       * Set volume level.
       *
       * The <b>level</b> is a linear factor being 1.0f by default, currently
-      * ranges from 0.0f to 256.0f due to internal mixing limitations.
+      * ranges from 0.0f to 64.0f due to internal mixing limitations.
       */
      DFBResult (*SetVolume) (
           IFusionSoundPlayback     *thiz,
@@ -734,18 +828,34 @@ DEFINE_INTERFACE( IFusionSoundPlayback,
       * Set pitch value.
       *
       * The <b>value</b> is a linear factor being 1.0f by default, currently
-      * ranges from 0.0f to 256.0f due to internal mixing limitations.
+      * ranges from 0.0f to 64.0f due to internal mixing limitations.
       */
      DFBResult (*SetPitch) (
           IFusionSoundPlayback     *thiz,
           float                     value
      );
+     
+     /*
+      * Set the direction of the playback.
+      */
+     DFBResult (*SetDirection) (
+          IFusionSoundPlayback     *thiz,
+          FSPlaybackDirection       direction
+     );
 )
 
+/*
+ * Result of a FMBufferCallback.
+ */
+typedef enum {
+     FMBCR_OK            = 0x00000000, /* Continue. */
+     FMBCR_BREAK         = 0x00000001, /* Stop loading. */
+} FMBufferCallbackResult;
+
 /*
  * Called after each buffer write.
  */
-typedef void (*FMBufferCallback)( int length, void *ctx );
+typedef FMBufferCallbackResult (*FMBufferCallback)( int length, void *ctx );
 
 /*
  * <i>No summary yet...</i>
@@ -759,7 +869,7 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
       */
      DFBResult (*GetCapabilities) (
           IFusionSoundMusicProvider   *thiz,
-          FSMusicProviderCapabilities *caps
+          FSMusicProviderCapabilities *ret_caps
      );
 
      /*
@@ -789,7 +899,7 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
       */
      DFBResult (*GetTrackDescription) (
           IFusionSoundMusicProvider *thiz,
-          FSTrackDescription        *desc
+          FSTrackDescription        *ret_desc
      );
 
      /*
@@ -798,7 +908,7 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
       */
      DFBResult (*GetStreamDescription) (
           IFusionSoundMusicProvider *thiz,
-          FSStreamDescription       *desc
+          FSStreamDescription       *ret_desc
      );
      
      /*
@@ -807,7 +917,7 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
       */
      DFBResult (*GetBufferDescription) (
           IFusionSoundMusicProvider *thiz,
-          FSBufferDescription       *desc
+          FSBufferDescription       *ret_desc
      );
 
    /** Playback **/
@@ -843,7 +953,7 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
           IFusionSoundMusicProvider *thiz,
           IFusionSoundBuffer        *destination,
           FMBufferCallback           callback,
-          void                      *ctx
+          void                      *callbackdata
      );
 
      /*
@@ -852,6 +962,14 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
      DFBResult (*Stop) (
           IFusionSoundMusicProvider *thiz
      );
+     
+     /*
+      * Get playback status.
+      */
+     DFBResult (*GetStatus) (
+          IFusionSoundMusicProvider *thiz,
+          FSMusicProviderStatus     *ret_status
+     );
 
    /** Media Control **/
 
@@ -865,13 +983,10 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
 
      /*
       * Gets current position within the current track.
-      *
-      * Returns <i>DFB_EOF</i> if the playback
-      * of the current track is finished.
       */
      DFBResult (*GetPos) (
           IFusionSoundMusicProvider *thiz,
-          double                    *seconds
+          double                    *ret_seconds
      );
 
      /*
@@ -879,7 +994,17 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
       */
      DFBResult (*GetLength) (
           IFusionSoundMusicProvider *thiz,
-          double                    *seconds
+          double                    *ret_seconds
+     );
+     
+   /** Advanced Playback **/
+     
+     /*
+      * Set the flags controlling playback.
+      */
+     DFBResult (*SetPlaybackFlags) (
+          IFusionSoundMusicProvider    *thiz,
+          FSMusicProviderPlaybackFlags  flags
      );
 )
 
