@@ -24,6 +24,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 
@@ -46,7 +48,8 @@ Probe( IFusionSoundMusicProvider_ProbeContext *ctx );
 
 static DFBResult
 Construct( IFusionSoundMusicProvider *thiz,
-           const char                *filename );
+           const char                *filename,
+           DirectStream              *stream );
 
 #include <direct/interface_implementation.h>
 
@@ -357,9 +360,11 @@ TimidityBufferThread( DirectThread *thread, void *ctx )
 
      while (data->playing) {
           void *buf;
+          int   len;
 
-          if (data->buffer->Lock( data->buffer, &buf ) != DFB_OK)
+          if (data->buffer->Lock( data->buffer, &buf, &len, 0 ) != DFB_OK)
                break;
+          len /= 2;
           
           pthread_mutex_lock( &data->lock );
 
@@ -369,14 +374,16 @@ TimidityBufferThread( DirectThread *thread, void *ctx )
                break;
           }
 
-          Timidity_PlaySome( buf, data->length );
+          Timidity_PlaySome( buf, len );
 
           pthread_mutex_unlock( &data->lock );
 
           data->buffer->Unlock( data->buffer );
 
-          if (data->callback)
-               data->callback( data->length, data->ctx );
+          if (data->callback) {
+               if (data->callback( len, data->ctx ))
+                    data->playing = false;
+          }
      }
 
      return NULL;
@@ -556,30 +563,16 @@ static DFBResult IFusionSoundMusicProvider_Timidity_GetLength(
 static DFBResult
 Probe( IFusionSoundMusicProvider_ProbeContext *ctx )
 {
-     char buf[5];
-     FILE *file;
+     if (!memcmp( ctx->header, "MThd", 4 ))
+          return DFB_OK;
 
-     file = fopen( ctx->filename, "rb" );
-     if (!file)
-          return DFB_UNSUPPORTED;
-
-     memset( buf, 0, 5 );
-     if (fread( buf, 1, 4, file ) != 4) {
-          fclose( file );
-          return DFB_UNSUPPORTED;
-     }
-     if (!strstr( buf, "MThd" )) {
-          fclose( file );
-          return DFB_UNSUPPORTED;
-     }
-
-     fclose( file );
-
-     return DFB_OK;
+     return DFB_UNSUPPORTED;
 }
 
 static DFBResult
-Construct( IFusionSoundMusicProvider *thiz, const char *filename )
+Construct( IFusionSoundMusicProvider *thiz, 
+           const char                *filename,
+           DirectStream              *stream )
 {
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IFusionSoundMusicProvider_Timidity)
 
