@@ -29,9 +29,10 @@
 
 #include <fusionsound.h>
 
-#include <fusion/types.h>
+#include <fusion/object.h>
 
 #include <core/types_sound.h>
+#include <core/sound_device.h>
 
 /*
  * Core initialization and deinitialization
@@ -45,14 +46,24 @@ DFBResult fs_core_destroy( CoreSound  *core, bool emergency );
 CoreSoundBuffer *fs_core_create_buffer  ( CoreSound *core );
 CorePlayback    *fs_core_create_playback( CoreSound *core );
 
+/*
+ * Object enumeration
+ */
+DirectResult     fs_core_enum_buffers   ( CoreSound            *core,
+                                          FusionObjectCallback  callback,
+                                          void                 *ctx );
+DirectResult     fs_core_enum_playbacks ( CoreSound            *core,
+                                          FusionObjectCallback  callback,
+                                          void                 *ctx );
+
 /*
  * Playback list management
  */
-DirectResult fs_core_playlist_lock( CoreSound *core );
+DirectResult fs_core_playlist_lock  ( CoreSound *core );
 DirectResult fs_core_playlist_unlock( CoreSound *core );
 
-DFBResult fs_core_add_playback( CoreSound    *core,
-                                CorePlayback *playback );
+DFBResult fs_core_add_playback   ( CoreSound    *core,
+                                   CorePlayback *playback );
 
 DFBResult fs_core_remove_playback( CoreSound    *core,
                                    CorePlayback *playback );
@@ -72,5 +83,15 @@ FusionWorld *fs_core_world( CoreSound *core );
  */
 FusionSHMPoolShared *fs_core_shmpool( CoreSound *core );
 
-#endif
+/*
+ * Returns device information.
+ */ 
+FSDeviceDescription *fs_core_device_description( CoreSound *core );
 
+/*
+ * Returns device configuration.
+ */
+CoreSoundDeviceConfig *fs_core_device_config( CoreSound *core );
+
+
+#endif
