@@ -34,6 +34,12 @@
 #include <core/fs_types.h>
 #include <core/types_sound.h>
 
+typedef enum {
+     CPS_NONE     = 0x00000000,
+     CPS_PLAYING  = 0x00000001,
+     CPS_LOOPING  = 0x00000002
+} CorePlaybackStatus;
+
 typedef enum {
      CPNF_START   = 0x00000001,
      CPNF_STOP    = 0x00000002,
@@ -50,10 +56,17 @@ typedef struct {
      int                             num;    /* Number of samples played (CPNF_ADVANCE) or zero. */
 } CorePlaybackNotification;
 
+
+/*
+ * Internal definitions.
+ */
+#define FS_PITCH_BITS  14
+#define FS_PITCH_ONE   (1 << FS_PITCH_BITS)
+
 /*
  * Creates a pool of playback objects.
  */
-FusionObjectPool *fs_playback_pool_create();
+FusionObjectPool *fs_playback_pool_create( const FusionWorld *world );
 
 /*
  * Generates fs_playback_ref(), fs_playback_attach() etc.
@@ -61,39 +74,44 @@ FusionObjectPool *fs_playback_pool_create();
 FUSION_OBJECT_METHODS( CorePlayback, fs_playback )
 
 
-DFBResult fs_playback_create      ( CoreSound        *core,
-                                    CoreSoundBuffer  *buffer,
-                                    bool              notify,
-                                    CorePlayback    **ret_playback );
+DFBResult fs_playback_create      ( CoreSound           *core,
+                                    CoreSoundBuffer     *buffer,
+                                    bool                 notify,
+                                    CorePlayback       **ret_playback );
 
-DFBResult fs_playback_enable      ( CorePlayback     *playback );
+DFBResult fs_playback_enable      ( CorePlayback        *playback );
 
-DFBResult fs_playback_start       ( CorePlayback     *playback,
-                                    bool              enable );
+DFBResult fs_playback_start       ( CorePlayback        *playback,
+                                    bool                 enable );
 
-DFBResult fs_playback_stop        ( CorePlayback     *playback,
-                                    bool              disable );
+DFBResult fs_playback_stop        ( CorePlayback        *playback,
+                                    bool                 disable );
 
-DFBResult fs_playback_set_stop    ( CorePlayback     *playback,
-                                    int               stop );
+DFBResult fs_playback_set_stop    ( CorePlayback        *playback,
+                                    int                  stop );
 
-DFBResult fs_playback_set_position( CorePlayback     *playback,
-                                    int               position );
+DFBResult fs_playback_set_position( CorePlayback        *playback,
+                                    int                  position );
 
-DFBResult fs_playback_set_volume  ( CorePlayback     *playback,
-                                    float             left,
-                                    float             right );
+DFBResult fs_playback_set_volume  ( CorePlayback        *playback,
+                                    float                left,
+                                    float                right );
 
-DFBResult fs_playback_set_pitch   ( CorePlayback     *playback,
-                                    int               pitch );
+DFBResult fs_playback_set_pitch   ( CorePlayback        *playback,
+                                    int                  pitch );
+
+DFBResult fs_playback_get_status  ( CorePlayback        *playback,
+                                    CorePlaybackStatus  *ret_status,
+                                    int                 *ret_position );
 
 /*
- * Internally called by core_audio.c in the audio thread.
+ * Internally called by core_sound.c in the audio thread.
  */
-DFBResult fs_playback_mixto       ( CorePlayback     *playback,
-                                    __fsf            *dest,
-                                    int               dest_rate,
-                                    int               max_samples );
+DFBResult fs_playback_mixto       ( CorePlayback        *playback,
+                                    __fsf               *dest,
+                                    int                  dest_rate,
+                                    int                  max_samples,
+                                    int                 *ret_samples );
+
 
 #endif
-
