@@ -24,6 +24,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <pthread.h>
 
 #include <fusionsound.h>
@@ -48,12 +50,12 @@ typedef struct {
 
      float                  volume;
      float                  pan;
+     
+     int                    pitch;
+     int                    dir;
 
      pthread_mutex_t        lock;
      pthread_cond_t         wait;
-     bool                   playing;
-     bool                   looping;
-     int                    position;
 } IFusionSoundPlayback_data;
 
 /******/
@@ -160,40 +162,60 @@ IFusionSoundPlayback_Continue( IFusionSoundPlayback *thiz )
 static DFBResult
 IFusionSoundPlayback_Wait( IFusionSoundPlayback *thiz )
 {
+     DFBResult ret;
+
      DIRECT_INTERFACE_GET_DATA(IFusionSoundPlayback)
 
      D_DEBUG( "%s (%p)\n", __FUNCTION__, data->playback );
 
      pthread_mutex_lock( &data->lock );
 
-     while (data->playing) {
-          if (data->looping) {
-               pthread_mutex_unlock( &data->lock );
-               return DFB_UNSUPPORTED;
-          }
+     for (;;) {
+          CorePlaybackStatus status;
 
-          pthread_cond_wait( &data->wait, &data->lock );
+          ret = fs_playback_get_status( data->playback, &status, NULL );
+          if (ret)
+               break;
+
+          if (status & CPS_PLAYING) {
+               if (status & CPS_LOOPING) {
+                    ret = DFB_UNSUPPORTED;
+                    break;
+               }
+               else
+                    pthread_cond_wait( &data->wait, &data->lock );
+          }
+          else
+               break;
      }
 
      pthread_mutex_unlock( &data->lock );
 
-     return DFB_OK;
+     return ret;
 }
 
 static DFBResult
 IFusionSoundPlayback_GetStatus( IFusionSoundPlayback *thiz,
-                                DFBBoolean           *playing,
-                                int                  *position )
+                                DFBBoolean           *ret_playing,
+                                int                  *ret_position )
 {
+     DFBResult          ret;
+     CorePlaybackStatus status;
+     int                position;
+
      DIRECT_INTERFACE_GET_DATA(IFusionSoundPlayback)
 
      D_DEBUG( "%s (%p)\n", __FUNCTION__, data->playback );
 
-     if (playing)
-          *playing = data->playing;
+     ret = fs_playback_get_status( data->playback, &status, &position );
+     if (ret)
+          return ret;
 
-     if (position)
-          *position = data->position;
+     if (ret_playing)
+          *ret_playing = (status & CPS_PLAYING) ? DFB_TRUE : DFB_FALSE;
+
+     if (ret_position)
+          *ret_position = position;
 
      return DFB_OK;
 }
@@ -206,8 +228,11 @@ IFusionSoundPlayback_SetVolume( IFusionSoundPlayback *thiz,
 
      D_DEBUG( "%s (%p, %.3f)\n", __FUNCTION__, data->playback, level );
 
-     if (level < 0.0f || level > 256.0f)
+     if (level < 0.0f)
           return DFB_INVARG;
+          
+     if (level > 64.0f)
+          return DFB_UNSUPPORTED;
 
      data->volume = level;
 
@@ -238,14 +263,41 @@ IFusionSoundPlayback_SetPitch( IFusionSoundPlayback *thiz,
 
      D_DEBUG( "%s (%p, %.3f)\n", __FUNCTION__, data->playback, value );
 
-     if (value < 0.0f || value > 256.0f)
+     if (value < 0.0f)
           return DFB_INVARG;
+          
+     if (value > 64.0f)
+          return DFB_UNSUPPORTED;
+          
+     data->pitch = (value * FS_PITCH_ONE + 0.5f);
 
-     fs_playback_set_pitch( data->playback, 0xffff * value / 256.0f );
+     fs_playback_set_pitch( data->playback, data->pitch*data->dir );
 
      return DFB_OK;
 }
 
+static DFBResult
+IFusionSoundPlayback_SetDirection( IFusionSoundPlayback *thiz,
+                                   FSPlaybackDirection   direction )
+{
+     DIRECT_INTERFACE_GET_DATA(IFusionSoundPlayback)
+     
+     D_DEBUG( "%s (%p, %d)\n", __FUNCTION__, data->playback, direction );
+     
+     switch (direction) {
+          case FSPD_FORWARD:
+          case FSPD_BACKWARD:
+               data->dir = direction;
+               break;
+          default:
+               return DFB_INVARG;
+     }
+     
+     fs_playback_set_pitch( data->playback, data->pitch*data->dir );
+     
+     return DFB_OK;
+}
+
 /******/
 
 DFBResult
@@ -279,25 +331,28 @@ IFusionSoundPlayback_Construct( IFusionSoundPlayback *thiz,
      data->stream   = (length < 0);
      data->length   = length;
      data->volume   = 1.0f;
+     data->pitch    = FS_PITCH_ONE;
+     data->dir      = +1;
 
      /* Initialize lock and condition. */
      direct_util_recursive_pthread_mutex_init( &data->lock );
      pthread_cond_init( &data->wait, NULL );
 
      /* Initialize method table. */
-     thiz->AddRef    = IFusionSoundPlayback_AddRef;
-     thiz->Release   = IFusionSoundPlayback_Release;
+     thiz->AddRef       = IFusionSoundPlayback_AddRef;
+     thiz->Release      = IFusionSoundPlayback_Release;
 
-     thiz->Start     = IFusionSoundPlayback_Start;
-     thiz->Stop      = IFusionSoundPlayback_Stop;
-     thiz->Continue  = IFusionSoundPlayback_Continue;
-     thiz->Wait      = IFusionSoundPlayback_Wait;
+     thiz->Start        = IFusionSoundPlayback_Start;
+     thiz->Stop         = IFusionSoundPlayback_Stop;
+     thiz->Continue     = IFusionSoundPlayback_Continue;
+     thiz->Wait         = IFusionSoundPlayback_Wait;
+ 
+     thiz->GetStatus    = IFusionSoundPlayback_GetStatus;
 
-     thiz->GetStatus = IFusionSoundPlayback_GetStatus;
-
-     thiz->SetVolume = IFusionSoundPlayback_SetVolume;
-     thiz->SetPan    = IFusionSoundPlayback_SetPan;
-     thiz->SetPitch  = IFusionSoundPlayback_SetPitch;
+     thiz->SetVolume    = IFusionSoundPlayback_SetVolume;
+     thiz->SetPan       = IFusionSoundPlayback_SetPan;
+     thiz->SetPitch     = IFusionSoundPlayback_SetPitch;
+     thiz->SetDirection = IFusionSoundPlayback_SetDirection;
 
      return DFB_OK;
 }
@@ -320,23 +375,9 @@ IFusionSoundPlayback_React( const void *msg_data,
      if (notification->flags & CPNF_ADVANCE)
           D_DEBUG( "%s: playback advanced to position %d\n", __FUNCTION__, notification->pos );
 
-     pthread_mutex_lock( &data->lock );
-
-     data->position = notification->pos;
-
-     if (notification->flags & (CPNF_START | CPNF_ADVANCE)) {
-          data->playing = true;
-          data->looping = notification->stop < 0 ? true : false;
-     }
-
-     if (notification->flags & CPNF_STOP) {
-          data->playing = false;
-          data->looping = false;
-     }
-
-     pthread_cond_broadcast( &data->wait );
-
-     pthread_mutex_unlock( &data->lock );
+     /* Notify any Wait()er on start/stop. */
+     if (notification->flags & (CPNF_START | CPNF_STOP))
+          pthread_cond_broadcast( &data->wait );
 
      return RS_OK;
 }
@@ -356,12 +397,12 @@ IFusionSoundPlayback_UpdateVolume( IFusionSoundPlayback_data* data )
 
      if (data->volume != 1.0f) {
           left *= data->volume;
-          if (left > 256.0f)
-               left = 256.0f;
+          if (left > 64.0f)
+               left = 64.0f;
 
           right *= data->volume;
-          if (right > 256.0f)
-               right = 256.0f;
+          if (right > 64.0f)
+               right = 64.0f;
      }
 
      return fs_playback_set_volume( data->playback, left, right );
